#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sat Dec  3 23:10:05 2016

@author: cham
"""

#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed Nov 23 17:37:24 2016

@author: cham
"""

#%%
import os
import sys
import re
import numpy as np
import ccdproc

from collections import OrderedDict
from astropy.table import Table, Column, Row
from hrs.logtable import log2table
from hrs.aperture import find_apertures, combine_apertures, group_apertures, extract_1dspec, combine_flat, substract_scattered_light
from hrs.calibration import fix_thar_sat_neg, thar_corr2d, interpolate_wavelength_shift, interpolate_order_shift, load_thar_temp
import hrs
#from hrs import *
import hrs
reload(hrs)
from joblib import Parallel, delayed


from scipy.interpolate import interp1d
from skimage.filters import gaussian

import glob
from astropy.io import fits

#%matplotlib qt
import matplotlib.pyplot as plt
from matplotlib import rcParams
rcParams.update({'font.size':20})

from tqdm import tqdm,trange

#%%

day = 20161118
print(day)
os.chdir('/hydrogen/song/star_spec/%s/night/raw/' % day)

#%%


    

fps_all = glob.glob('./*.fits')
fps_all.sort()
t = scan_fits_header(fps_all)

t.add_column(Column(np.zeros((len(t),), dtype=int)*np.nan, 'xdrift'))
t.add_column(Column(np.zeros((len(t),), dtype=bool), 'astemplate'))
#t.show_in_browser()

#%%

""" check Y shift """
ind_bias = t['IMAGETYP'] == 'BIAS'
print("%s BIAS found" % np.sum(ind_bias))
#bias = ccdproc.combine(','.join(t['fps'][ind_bias]), unit='adu', method='average')

ind_flat = t['IMAGETYP'] == 'FLAT'
print("%s FLAT found" % np.sum(ind_flat))
#flat = ccdproc.combine(','.join(t['fps'][ind_flat]), unit='adu', method='median')

ind_flati2 = t['IMAGETYP'] == 'FLATI2'
print("%s FLATI2 found" % np.sum(ind_flati2))
#flat = ccdproc.combine(','.join(t['fps'][ind_flat]), unit='adu', method='median')

ind_thar = t['IMAGETYP'] == 'THAR'
print("%s THAR found" % np.sum(ind_thar))
#thar = ccdproc.combine(','.join(t['fps'][ind_thar]), unit='adu', method='median')

ind_star = t['IMAGETYP'] == 'STAR'
print("%s STAR found" % np.sum(ind_star))
#flat_bias = ccdproc.subtract_bias(flat, bias)

#%%

""" Y shift """

sub_flat = np.where(ind_flat)[0]
sub_flati2 = np.where(ind_flati2)[0]
sub_star = np.where(ind_star)[0]
sub_thar = np.where(ind_thar)[0]
sub_bias = np.where(ind_bias)[0]

if len(sub_flat)>0:
    t['astemplate'][sub_flat[0]] = True
    flat_temp = ccdproc.CCDData.read(t['fps'][sub_flat[0]], unit='adu')
else:
    t['astemplate'][sub_star[0]] = True
    flat_temp = ccdproc.CCDData.read(t['fps'][sub_star[0]], unit='adu')
    
if len(sub_bias)>0:
    t['astemplate'][sub_bias[0]] = True
    bias_temp = ccdproc.CCDData.read(t['fps'][sub_bias[0]], unit='adu')
else:
    bias_temp = ccdproc.CCDData(np.ones((2048, 2048))*300, unit='adu')
flat_temp = ccdproc.subtract_bias(flat_temp, bias_temp)


#%%    

fig = plt.figure(figsize=(10, 5))
ax = fig.add_subplot(111)

sub = sub_flati2    
print("----- FLATI2 -----")
if len(sub)>0:
    drifts = measure_xshift_2dcorr(t['fps'][sub], bias_temp, flat_temp, xmax=12)
    plt.plot(t['MJD-MID'][sub], drifts, 'o', ms=20, mec='r', mfc='None', label='FLATI2')
    for i, this_drift in zip(sub, drifts): 
        t['xdrift'][i] = this_drift
    
sub = sub_flat
print("------ FLAT ------")
if len(sub)>0:
    drifts = measure_xshift_2dcorr(t['fps'][sub], bias_temp, flat_temp, xmax=12)
    plt.plot(t['MJD-MID'][sub], drifts, 'o', ms=20, mec='b', mfc='None', label='FLAT')
    for i, this_drift in zip(sub, drifts): 
        t['xdrift'][i] = this_drift
        
sub = sub_thar
print("------ THAR ------")
if len(sub)>0:
    drifts = measure_xshift_2dcorr(t['fps'][sub], bias_temp, flat_temp, xmax=12)
    plt.plot(t['MJD-MID'][sub], drifts, '^', ms=20, mec='k', mfc='None', label='THAR')
    for i, this_drift in zip(sub, drifts): 
        t['xdrift'][i] = this_drift
        
sub = sub_star
print("------ STAR ------")
if len(sub)>0:
    drifts = measure_xshift_2dcorr(t['fps'][sub], bias_temp, flat_temp, xmax=12)
    plt.plot(t['MJD-MID'][sub], drifts, 's-', c='c', ms=20, mec='c', mfc='None', label='STAR')
    for i, this_drift in zip(sub, drifts): 
        t['xdrift'][i] = this_drift
        
plt.grid()
l = plt.legend()
l.set_frame_on(False)
#fig.set_figheight(6)
#fig.set_figwidth(12)
plt.xlabel('MJD-MID (date:%s)'%day)
plt.ylabel("DRIFT/pixel")
fig.tight_layout()

ylim = ax.get_ylim()
plt.ylim(ylim[0]-2, ylim[1]+2)

#%%
figpath_pdf = "/hydrogen/song/figs/Xdrift_%s.pdf" % day
figpath_png = "/hydrogen/song/figs/Xdrift_%s.png" % day

fig.savefig(figpath_pdf)
print(figpath_pdf)
fig.savefig(figpath_png)
print(figpath_png)

#%%
figpath_fits = "/hydrogen/song/figs/t_%s.fits" % day

for i, colname in enumerate(t.colnames):
    if t[colname].dtype==np.dtype('O'):
        newcol = t[colname].astype('str')
        t.remove_column(colname)
        t.add_column(newcol, i)

t.write(figpath_fits, format="fits", overwrite=True)



# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Wed Nov 23 17:37:24 2016

Modifications
-------------
-

Aims
----
- utils for reading HRS log files

"""

import re
from collections import OrderedDict

import numpy as np
from astropy.table import Table, Column


class LogTable(Table):

    @staticmethod
    def read_log(filepath, verbose=False):
        """ read log table

        Examples
        --------
        >>> t = LogTable.read_log("./20161212.txt", True)

        Parameters
        ----------
        filepath: string
            log table file path
        verbose: bool
            if True, print details

        Returns
        -------
        LogTable instance

        """
        return LogTable(log2table(filepath, verbose=verbose))


def log2table(filepath, verbose=False):
    """ process observation log file

    Parameters
    ----------
    filepath: string
        file path of observation log

    """

    # 0. read log data
    f = open(filepath)
    slist = f.readlines()
    sjoin = "".join(slist)
    f.close()

    # 1. extract meta data
    observer = re.search(r"(?<=Observer:)[a-z\s]{1,30}(?=[\t|\r|\n|,])",
                         sjoin, re.I | re.M).group(0).strip()
    institute = re.search(r"(?<=Institute:)[a-z\s]{1,30}(?=[\t|\r|\n|,])",
                          sjoin, re.I | re.M).group(0).strip()
    night_assistant = re.search(
        r"(?<=Night Assistant:)[a-z\s]{1,30}(?=[\t|\r|\n|,])",
        sjoin, re.I | re.M).group(0).strip()
    date = re.search(r"(?<=Date):\d+-\d+-\d+(?=[\t|\r|\n|,])",
                     sjoin, re.I | re.M).group(0).strip()
    seeing = re.search(r"(?<=Seeing):\d+\.\d*\"(?=[\t|\r|\n|,])",
                       sjoin, re.I | re.M).group(0).strip()
    weather = re.search(r"(?<=Weather:).+(?=[\t|\r|\n|,])",
                        sjoin, re.I | re.M).group(0).strip()
    rh = re.search(r"(?<=RH:)[0-9-]+%", sjoin, re.I | re.M).group(0).strip()

    log_meta = OrderedDict(observer=observer, institute=institute,
                           night_assistant=night_assistant, date=date,
                           seeing=seeing, weather=weather, rh=rh)
    if verbose:
        print("@Cham: the meta data")
        print(log_meta)

    # 2.extract log items
    # 2.1 find items | this is where log items start
    for i, s in enumerate(slist):
        if s.find("File") != -1:
            break
    iframe = i + 1
    if verbose:
        print("@Cham: log items start from LINE %s" % iframe)

    # 2.2 initialize table
    colnames = [
        "file", "obj", "exp_start", "exp_time", "RA", "Dec", "epoch", "note"]
    coltypes = [
        "|S50", "|S50", "|S50", "|S50", "|S50", "|S50", "|S50", "|S100"]
    cols_init = [Column(np.array([], dtype=coltype), colname) for
                 colname, coltype in zip(colnames, coltypes)]
    log_tbl = Table(cols_init)
    if verbose:
        log_tbl.pprint()
        print("@Cham: table initialized successfully!")

    # 2.3 iteratively extract log items
    for i in np.arange(iframe, len(slist)):
        # split string & extract infomation
        contents = re.split(r"\s*", slist[i].strip())

        # convert system frames to lower case
        if contents[1].lower() in {"bias", "flat", "thar"}:
            contents[1] = contents[1].lower()

        # add rows in table
        if contents[0].find("-") != -1:
            # if there are multi frames, add multi items
            contents0 = re.split(r"-", contents[0])
            file_start = np.int(contents0[0])
            file_stop = np.int(
                contents0[0][:-len(contents0[1])] + contents0[1])
            for i_file in np.arange(file_start, file_stop + 1):
                new_row = fill_list_to_len(
                    ["%s" % i_file] + contents[1:], len(log_tbl.colnames))
                log_tbl.add_row(new_row)
        else:
            # if there is only one frame, add one item
            new_row = fill_list_to_len(contents, len(log_tbl.colnames))
            log_tbl.add_row(new_row)

    # 3. add meta data to table
    log_tbl.meta = log_meta
    if verbose:
        print("@Cham: log table substantiated successfully!")

    # 4. perfect table
    return perfect_logtable(log_tbl)


def fill_list_to_len(slist, lgoal, tobefilled=""):
    """ fill a list with something to a given length """
    for i in range(lgoal - len(slist)):
        slist.append(tobefilled)
    return slist


def perfect_logtable(log_tbl, fill0=True):
    """ perfect log table """
    # convert exp_time to float
    exptime = log_tbl["exp_time"]
    log_tbl.remove_column("exp_time")
    log_tbl.add_column(exptime.astype(np.float), 3)

    # add filename column
    filename_data = []
    if fill0:
        for _ in log_tbl["file"]:
            m = np.int(_) % 1000
            if m > 99:
                filename_data.append(_[:8] + "00" + _[8:] + ".fits")
            elif m>9:
                filename_data.append(_[:8] + "0" + _[8:] + ".fits")
            else:
                filename_data.append(_ + ".fits")
    else:
        for _ in log_tbl["file"]:
            filename_data.append(_ + ".fits")
    filename = Column(filename_data, "filename")

    log_tbl.add_column(filename)

    return log_tbl


def test():
    return log2table("/town/HRS/20161110test/20161110.txt")


if __name__ == "__main__":
    x = test()
    x.pprint()
    # x.write("/town/HRS/20161110test/20161110.fits")



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sat Dec  3 23:10:05 2016

@author: cham
"""

import sys

import ccdproc
import matplotlib.pyplot as plt
import numpy as np
from astropy.io import fits
from astropy.table import Table, Column
from matplotlib import rcParams
from tqdm import trange

from hrs import thar_corr2d

from .utils import scan_files, scan_flux90

rcParams.update({"font.size": 20})


def measure_xshift_2dcorr(imfps, bias_temp, flat_temp, xmax=12):
    drifts = np.zeros(len(imfps))
    cfn = sys._getframe().f_code.co_name
    for i in trange(len(imfps), ncols=100, ascii=False, desc="**" + cfn + "**",
                    unit=" file"):
        data = ccdproc.CCDData.read(imfps[i], unit="adu")
        data = ccdproc.subtract_bias(data, bias_temp)
        shift, corr2d = thar_corr2d(data.data, flat_temp.data,
                                    xtrim=(512, 1536), ytrim=(512, 1536),
                                    x_shiftmax=xmax, y_shiftmax=0,
                                    verbose=False)
        drifts[i] = np.int(shift[0])
    return drifts


def check_xdrift(t):
    # check Y shift
    ind_bias = t["IMAGETYP"] == "BIAS"
    print("%s BIAS found" % np.sum(ind_bias))
    ind_flat = t["IMAGETYP"] == "FLAT"
    print("%s FLAT found" % np.sum(ind_flat))
    ind_flati2 = t["IMAGETYP"] == "FLATI2"
    print("%s FLATI2 found" % np.sum(ind_flati2))
    ind_thar = t["IMAGETYP"] == "THAR"
    print("%s THAR found" % np.sum(ind_thar))
    ind_star = t["IMAGETYP"] == "STAR"
    print("%s STAR found" % np.sum(ind_star))

    # Y shift
    sub_flat = np.where(ind_flat)[0]
    sub_flati2 = np.where(ind_flati2)[0]
    sub_star = np.where(ind_star)[0]
    sub_thar = np.where(ind_thar)[0]
    sub_bias = np.where(ind_bias)[0]

    if len(sub_flat) > 0:
        # if there is a flat
        t["astemplate"][sub_flat[0]] = True
        flat_temp = ccdproc.CCDData.read(t["fps"][sub_flat[0]], unit="adu")
    else:
        # no flat, use star instead
        t["astemplate"][sub_star[0]] = True
        flat_temp = ccdproc.CCDData.read(t["fps"][sub_star[0]], unit="adu")

    if len(sub_bias) > 0:
        # if there is a bias
        t["astemplate"][sub_bias[0]] = True
        bias_temp = ccdproc.CCDData.read(t["fps"][sub_bias[0]], unit="adu")
    else:
        # use 300 as bias instead
        bias_temp = ccdproc.CCDData(np.ones((2048, 2048)) * 300, unit="adu")
    flat_temp = ccdproc.subtract_bias(flat_temp, bias_temp)

    # plot figure
    fig = plt.figure(figsize=(11, 5))
    ax = fig.add_subplot(111)

    sub = sub_flati2
    print("----- FLATI2 -----")
    if len(sub) > 0:
        drifts = measure_xshift_2dcorr(t["fps"][sub], bias_temp, flat_temp,
                                       xmax=12)
        ax.plot(t["MJD-MID"][sub], drifts, "o", ms=20, mec="r", mfc="None",
                label="FLATI2x%s" % len(sub))
        for i, this_drift in zip(sub, drifts):
            t["xdrift"][i] = this_drift

    sub = sub_flat
    print("------ FLAT ------")
    if len(sub) > 0:
        drifts = measure_xshift_2dcorr(t["fps"][sub], bias_temp, flat_temp,
                                       xmax=12)
        ax.plot(t["MJD-MID"][sub], drifts, "o", ms=20, mec="b", mfc="None",
                label="FLATx%s" % len(sub))
        for i, this_drift in zip(sub, drifts):
            t["xdrift"][i] = this_drift

    sub = sub_thar
    print("------ THAR ------")
    if len(sub) > 0:
        drifts = measure_xshift_2dcorr(t["fps"][sub], bias_temp, flat_temp,
                                       xmax=12)
        ax.plot(t["MJD-MID"][sub], drifts, "^", ms=20, mec="k", mfc="None",
                label="THARx%s" % len(sub))
        for i, this_drift in zip(sub, drifts):
            t["xdrift"][i] = this_drift

    sub = sub_star
    print("------ STAR ------")
    if len(sub) > 0:
        drifts = measure_xshift_2dcorr(t["fps"][sub], bias_temp, flat_temp,
                                       xmax=12)
        ax.plot(t["MJD-MID"][sub], drifts, "s-", c="c", ms=20, mec="c",
                mfc="None", label="STARx%s" % len(sub))
        for i, this_drift in zip(sub, drifts):
            t["xdrift"][i] = this_drift

    # save figure
    plt.grid()
    l = plt.legend()
    l.set_frame_on(False)
    # fig.set_figheight(6)
    # fig.set_figwidth(12)

    plt.xlabel("MJD-MID")
    plt.ylabel("DRIFT/pixel")
    fig.tight_layout()

    ylim = ax.get_ylim()
    plt.ylim(ylim[0] - 2, ylim[1] + 2)
    xlim = ax.get_xlim()
    plt.xlim(xlim[0], xlim[0] + 1.4)

    return t, fig

#
# figpath_pdf = "/hydrogen/song/figs/Xdrift_%s.pdf" % day
# figpath_png = "/hydrogen/song/figs/Xdrift_%s.png" % day
#
# fig.savefig(figpath_pdf)
# print(figpath_pdf)
# fig.savefig(figpath_png)
# print(figpath_png)
#
# # %%
# figpath_fits = "/hydrogen/song/figs/t_%s.fits" % day
#
# t.write(figpath_fits, format="fits", overwrite=True)



# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Sun Jan  8 14:00:00 2016

Modifications
-------------
-

Aims
----
- utils for scanning SONG files

"""


import glob
import sys

import numpy as np
from astropy.io import fits
from astropy.table import Table, Column
from tqdm import trange
import ccdproc
from joblib import Parallel, delayed


# scan fits header
def scan_fits_header(fps):
    cfn = sys._getframe().f_code.co_name
    for i in trange(len(fps), ncols=100, ascii=False, desc="**" + cfn + "**",
                    unit=" file"):
        fp = fps[i]
        #        print i, len(fps)
        # read fits
        hl = fits.open(fp)
        hdr_dict = dict(hl[0].header)

        # initiate table
        if i == 0:
            t = Table()
            for k, v in hdr_dict.items():
                #                print k, v
                if type(v) is str:
                    t.add_column(
                        Column(data=np.array([]), dtype=object, name=k))
                else:
                    t.add_column(
                        Column(data=np.array([]), dtype=type(v), name=k))
        # add rows
        t.add_row(hdr_dict.values())
    t.add_column(Column(fps, "fps"), 0)

    # fix column dtype
    for i, colname in enumerate(t.colnames):
        if t[colname].dtype == np.dtype("O"):
            newcol = t[colname].astype("str")
            t.remove_column(colname)
            t.add_column(newcol, i)

    return t


def scan_files(dirpath, xdriftcol=True):

    fps_all = glob.glob(dirpath+"/*.fits")
    fps_all.sort()
    t = scan_fits_header(fps_all)

    if xdriftcol:
        t.add_column(Column(np.zeros((len(t),), dtype=int) * np.nan, "xdrift"))
        t.add_column(Column(np.zeros((len(t),), dtype=bool), "astemplate"))

    return t


def scan_flux90(t, ind_scan=None, ind_type='bool', pct=90,
                n_jobs=-1, verbose=10, unit='adu', **kwargs):
    """

    Parameters
    ----------
    t: table
        catalog of files
    ind_scan: bool array

    ind_type: string
        {'bool', 'int'}
    pct: float
        default is 90
    n_jobs: int
        number of processes launched
    verbose:
        verbose level
    kwargs:
        ccdproc.CCDData.read() keywords

    Returns
    -------
    f90: array
        the 90th percentile array

    """
    if ind_scan is None:
        ind_scan = np.ones((len(t),), dtype=bool)
        ind_type = 'bool'

    f90_ = Parallel(n_jobs=n_jobs, verbose=verbose)(
        delayed(scan_flux90_)(fp, pct, **kwargs) for fp in t['fps'][ind_scan])

    if ind_type == 'bool':
        f90 = np.array(f90_)*ind_scan
    else:
        f90 = np.zeros((len(t),), dtype=float)
        for i90_, i90 in enumerate(ind_scan):
            f90[i90] = f90_[i90_]

    return f90


def scan_flux90_(fp, pct, **kwargs):
    return np.nanpercentile(ccdproc.CCDData.read(fp, **kwargs), pct)


# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Wed Jan   4 14:00:00 2016

Modifications
-------------
-

Aims
----
- to modify ccdproc package

"""

import numpy as np
import ccdproc
from ccdproc import *

__version__ = ccdproc.__version__


class CCDData(ccdproc.CCDData):
    def rot90(self, k):
        """ Rotate self by 90 degrees in the counter-clockwise direction.

        Parameters
        ----------
        k : integer
            Number of times the array is rotated by 90 degrees.

        """
        self.data = np.rot90(self.data, k)
        return self


def combine(*args, **kwargs):
    return CCDData(ccdproc.combine(*args, **kwargs))



# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Mon Nov 28 15:00:00 2016

Modifications
-------------
-

Aims
----
- utils for calibration

"""

import itertools

import numpy as np
from astropy.io import fits
from joblib import Parallel, delayed
from scipy.interpolate import splrep, splev
from scipy.optimize import curve_fit, leastsq


# ############################## #
#      fix thar spectra
# ############################## #

def fix_thar_sat_neg(thar1d, arm=20, sat_count=50000):
    # ind_sat_conv
    ind_sat = thar1d >= sat_count
    ind_sat_conv = np.zeros_like(ind_sat)
    x_sat, y_sat = np.where(ind_sat)
    for i in range(len(x_sat)):
        ind_sat_conv[x_sat[i], y_sat[i] - arm:y_sat[i] + arm] = 1
    # ind_neg
    ind_neg = thar1d < 0.
    # combine to ind_bad
    ind_bad = np.logical_or(ind_neg, ind_sat_conv)
    thar1d_fixed = np.where(ind_bad, 0, thar1d)

    return thar1d_fixed


# ############################## #
#      2d correlation
# ############################## #

def thar_corr2d(thar1d_fixed, thar_temp,xtrim=(1024, 3072), ytrim=(53, 73),
                x_shiftmax=20, y_shiftmax=5, verbose=False):
    """ determine the shift of *thar1d_fixed* relative to *thar_temp*

    Parameters
    ----------
    thar1d_fixed:
        image whose shift is to be determined
    thar_temp:
        reference image
    xtrim: tuple
        (xmin, xmax)
    ytrim: tuple
        (ymin, ymax)
    x_shiftmax: int
        the max y shift for correlation
    y_shiftmax: int
        the max y shift for correlation

    Returns
    -------
    (x_shift, y_shift), corr2d

    """
    # in case that the input data are int
    thar1d_fixed = np.array(thar1d_fixed).astype(float)
    thar_temp = np.array(thar_temp).astype(float)

    if verbose:
        print("@Cham: computing 2D cross-correlation...")

    # initialize result
    corr2d = np.zeros((1 + 2 * y_shiftmax, 1 + 2 * x_shiftmax))
    # make slice
    xslice_temp = slice(*xtrim)
    yslice_temp = slice(*ytrim)
    # 2D correlation
    for xofst in range(-x_shiftmax, x_shiftmax + 1, 1):
        for yofst in range(-y_shiftmax, y_shiftmax + 1, 1):
            xslice = slice(xtrim[0] + xofst, xtrim[1] + xofst, 1)
            yslice = slice(ytrim[0] + yofst, ytrim[1] + yofst, 1)
            corr2d[yofst + y_shiftmax, xofst + x_shiftmax] = \
                np.mean(thar1d_fixed[yslice, xslice] *
                        thar_temp[yslice_temp, xslice_temp])
    # select maximum value
    y_, x_ = np.where(corr2d == np.max(corr2d))
    x_shift = np.int(x_shiftmax - x_)  # reverse sign
    y_shift = np.int(y_shiftmax - y_)

    return (x_shift, y_shift), corr2d


# ############################## #
#      shift wave & order
# ############################## #

def interpolate_wavelength_shift(w, shift, thar_temp, thar1d_fixed):
    xshift, yshift = shift

    xcoord = np.arange(w.shape[1])
    ycoord = np.arange(w.shape[0])

    # for X, no difference, for Y, orders are different
    xcoord_xshift = xcoord + xshift
    ycoord_yshift = np.arange(
        w.shape[0] + thar1d_fixed.shape[0] - thar_temp.shape[0]) + yshift
    # shfit X
    w_x = np.zeros(w.shape)
    for i in range(w.shape[0]):
        s = splrep(xcoord, w[i], k=3, s=0)
        w_x[i] = splev(xcoord_xshift, s)

    # shift Y
    w_x_y = np.zeros((thar1d_fixed.shape[0], w.shape[1]))
    for j in range(w.shape[1]):
        s = splrep(ycoord, w_x[:, j], k=3, s=0)
        w_x_y[:, j] = splev(ycoord_yshift, s)

    return w_x_y


def interpolate_order_shift(order_temp, shift, thar1d_fixed):
    xshift, yshift = shift

    # for X, no difference, for Y, orders are different
    ycoord_yshift = np.arange(thar1d_fixed.shape[0]) + \
                    order_temp[0, 0] + yshift
    order_interp = np.repeat(ycoord_yshift.reshape(-1, 1),
                             thar1d_fixed.shape[1], axis=1)

    return order_interp


# ############################## #
#      load temp thar spectra
# ############################## #

def load_thar_temp(thar_temp_path):
    hl = fits.open(thar_temp_path)
    wave = hl['wave'].data
    thar = hl['thar'].data
    order = hl['order'].data
    return wave, thar, order


# ############################## #
#      refine thar positions
# ############################## #

def refine_thar_positions(wave_init, order_init, thar1d_fixed, thar_list,
                          fit_width=5., lc_tol=5., k=3, n_jobs=10, verbose=10):
    print("@Cham: refine ThAr positions ...")

    # refine thar positions for each order
    r = Parallel(n_jobs=n_jobs, verbose=verbose, batch_size=1)(
        delayed(refine_thar_positions_order)(
            wave_init[i_order],
            np.arange(wave_init.shape[1]),
            thar1d_fixed[i_order],
            thar_list[(thar_list > np.min(wave_init[i_order]) + 1.) * (
            thar_list < np.max(wave_init[i_order]) - 1.)],
            order_init[i_order, 0],
            fit_width=fit_width, lc_tol=lc_tol, k=k
        ) for i_order in range(wave_init.shape[0]))

    # remove all null values
    null_value = (None, None, None, None, None)
    for i in range(r.count(null_value)):
        r.remove(null_value)
        print(len(r))

    # collect data
    lc_coord = np.array(np.hstack([_[0] for _ in r]))
    lc_order = np.array(np.hstack([_[1] for _ in r]))
    lc_thar = np.array(np.hstack([_[2] for _ in r]))
    popt = np.array(np.vstack([_[3] for _ in r]))
    pcov = np.array(np.vstack([_[4] for _ in r]))

    return lc_coord, lc_order, lc_thar, popt, pcov


def refine_thar_positions_order(this_wave_init, this_xcoord, this_thar,
                                this_thar_list, this_order, fit_width=5.,
                                lc_tol=5., k=3):
    if len(this_thar_list) == 0:
        return None, None, None, None, None

    popt_list = []
    pcov_list = []

    # refine all thar positions in this order
    for i_thar_line, each_thar_line in enumerate(this_thar_list):

        # cut local spectrum
        ind_local = (this_wave_init > each_thar_line - fit_width) * (
            this_wave_init < each_thar_line + fit_width)

        # set bounds
        p0 = (0, 0, 1000., each_thar_line, .05)
        bounds = ((-np.inf, -np.inf, 0., each_thar_line - lc_tol, 0.001),
                  (+np.inf, +np.inf, np.inf, each_thar_line + lc_tol, 2.))

        try:
            popt, pcov = curve_fit(gauss_poly1, this_wave_init[ind_local],
                                   this_thar[ind_local], p0=p0, bounds=bounds)
            pcov = np.diagonal(pcov)
        except RuntimeError:
            popt = np.ones_like(p0) * np.nan
            pcov = np.ones((len(p0),)) * np.nan

        popt_list.append(popt)
        pcov_list.append(pcov)

    # interpolation for X corrdinates
    if np.all(np.diff(this_wave_init) >= 0):
        tck = splrep(this_wave_init, this_xcoord, k=k)
    elif np.all(np.diff(this_wave_init) <= 0):
        tck = splrep(this_wave_init[::-1], this_xcoord[::-1], k=k)
    else:
        raise (ValueError("@Cham: error occurs in interpolation!"))

    # lccov_list = np.array(lccov_list)
    popt_list = np.array(popt_list)
    pcov_list = np.array(pcov_list)

    lc_coord = splev(popt_list[:, 3], tck)
    lc_order = np.ones_like(lc_coord) * this_order

    return lc_coord, lc_order, this_thar_list, popt_list, pcov_list


# ############################## #
#      2D surface fit
# ############################## #

def polyval2d(x, y, coefs, orders=None):
    if orders is None:
        orderx, ordery = coefs.shape
    else:
        orderx, ordery = orders

    ij = itertools.product(range(orderx + 1), range(ordery + 1))
    z = np.zeros_like(x)
    for a, (i, j) in zip(coefs.flatten(), ij):
        if i + j < np.max((orderx, ordery)):
            #        print a,i,j
            z += a * x ** i * y ** j
    return z


def gauss_poly1(x, p0, p1, a, b, c):
    return p0 + p1 * x + a/np.sqrt(2.*np.pi)/c * np.exp(-0.5*((x - b) / c) ** 2.)


def residual_chi2(coefs, x, y, z, w, poly_order):
    return np.nansum(residual(coefs, x, y, z, w, poly_order) ** 2.)


def residual_lar(coefs, x, y, z, w, poly_order):
    fitted = polyval2d(x, y, coefs, poly_order)
    return np.sqrt(np.abs((fitted - z) * w))


def residual(coefs, x, y, z, w, poly_order):
    fitted = polyval2d(x, y, coefs, poly_order)
    return (fitted - z) * w


# ############################## #
#      standardization
# ############################## #

def standardize_inverse(x, xmean, xstd):
    return (x * xstd) + xmean


def standardize(x):
    return (x - np.nanmean(x)) / np.nanstd(x), np.nanmean(x), np.nanstd(x)


# ############################## #
#      fit grating equation
# ############################## #

def fit_grating_equation(lc_coord, lc_order, lc_thar, popt, pcov,
                         poly_order=(3, 5), max_dev_threshold=100, iter=False):
    # pick good thar lines
    try:
        ind_good_thar = np.isfinite(lc_coord) * (popt[:, 4] < 1.0) * (
            pcov[:, 3] < 1.0)
    except:
        ind_good_thar = np.ones_like(lc_coord, dtype=bool)

    lc_coord = lc_coord[ind_good_thar]
    lc_order = lc_order[ind_good_thar]
    lc_thar = lc_thar[ind_good_thar]

    # standardization
    lc_coord_s, lc_coord_mean, lc_coord_std = standardize(lc_coord)
    lc_order_s, lc_order_mean, lc_order_std = standardize(lc_order)
    ml_s, ml_mean, ml_std = standardize(lc_thar * lc_order)
    # scaler
    scaler_coord = lc_coord_mean, lc_coord_std
    scaler_order = lc_order_mean, lc_order_std
    scaler_ml = ml_mean, ml_std

    # weight
    weight = np.ones_like(ml_s)

    # fit surface
    x0 = np.zeros(poly_order)
    x0, ier = leastsq(residual_lar, x0,
                      args=(lc_coord_s, lc_order_s, ml_s, weight, poly_order))

    # iter
    if iter:
        n_loop = 0
        while True:
            n_loop += 1
            x_mini_lsq, ier = leastsq(residual_lar, x0, args=(
                lc_coord_s, lc_order_s, ml_s, weight, poly_order))
            fitted = polyval2d(lc_coord_s, lc_order_s, x_mini_lsq, poly_order)
            fitted_wave = standardize_inverse(fitted, ml_mean, ml_std) / lc_order
            fitted_wave_diff = fitted_wave - lc_thar

            ind_max_dev = np.nanargmax(
                np.abs(fitted_wave_diff - np.nanmedian(fitted_wave_diff)))
            if np.abs(fitted_wave_diff[ind_max_dev] - np.nanmedian(
                    fitted_wave_diff)) > max_dev_threshold:
                weight[ind_max_dev] = 0.
                ml_s[ind_max_dev] = np.nan
                lc_thar[ind_max_dev] = np.nan
                print("@Cham: [n_loop = %s] max_dev = %s" % (
                    n_loop, fitted_wave_diff[ind_max_dev]))
            else:
                if n_loop == 0:
                    print("@Cham: no points cut in iterarions ...")
                break
    else:
        x_mini_lsq, ier = leastsq(residual_lar, x0, args=(
            lc_coord_s, lc_order_s, ml_s, weight, poly_order))
        fitted = polyval2d(lc_coord_s, lc_order_s, x_mini_lsq, poly_order)
        fitted_wave = standardize_inverse(fitted, ml_mean, ml_std) / lc_order
        fitted_wave_diff = fitted_wave - lc_thar

        ind_kick = np.abs(fitted_wave_diff - np.nanmedian(fitted_wave_diff)) > max_dev_threshold
        weight[ind_kick] = 0.
        ml_s[ind_kick] = np.nan
        lc_thar[ind_kick] = np.nan
        x_mini_lsq, ier = leastsq(residual_lar, x0, args=(
            lc_coord_s, lc_order_s, ml_s, weight, poly_order))

    ind_good_thar = np.where(ind_good_thar)[0][np.isfinite(lc_thar)]

    return x_mini_lsq, ind_good_thar, scaler_coord, scaler_order, scaler_ml


def grating_equation_predict(grid_coord, grid_order, x_mini_lsq, poly_order,
                             scaler_coord, scaler_order, scaler_ml):
    sgrid_coord = (grid_coord - scaler_coord[0]) / scaler_coord[1]
    sgrid_order = (grid_order - scaler_order[0]) / scaler_order[1]

    sgrid_fitted = polyval2d(sgrid_coord.flatten(), sgrid_order.flatten(),
                             x_mini_lsq, poly_order)
    sgrid_fitted_wave = (sgrid_fitted * scaler_ml[1] + scaler_ml[0]) / \
                        grid_order.flatten()
    sgrid_fitted_wave = sgrid_fitted_wave.reshape(grid_coord.shape)
    return sgrid_fitted_wave



# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Wed Jan   4 14:00:00 2016

Modifications
-------------
-

Aims
----
- SONG echelle spectra extractor

"""

import numpy as np

from . import ccdproc_mod as ccdproc


def list_image(t, imagetp='FLAT', kwds=None, max_print=None):
    """ list images with specified IMAGETYP value

    Examples
    --------
    >>> extract.list_image(t2, imagetp='STAR', kwds=['OBJECT'])

    Parameters
    ----------
    t: Table
        catalog of files, generated by *scan_files*
    imagetp: string
        IMAGETYP value
    kwds: list
        optional. additional columns to be displayed
    max_print:
        max line number

    Returns
    -------

    """
    ind_mst = np.where(t['IMAGETYP'] == imagetp)[0]

    if max_print is not None:
        if max_print > len(ind_mst):
            max_print = len(ind_mst)
    else:
        max_print = len(ind_mst)

    print("@SONG: these are all images of type %s" % imagetp)
    print("+ --------------------------------------------------")
    if kwds is None or kwds == '':
        for i in range(max_print):
            print("+ %04d - %s" % (i, t['fps'][ind_mst[i]]))
    else:
        assert isinstance(kwds, list)
        for kwd in kwds:
            try:
                assert kwd in t.colnames
            except AssertionError:
                print('kwd', kwd)
                raise AssertionError()

        for i in range(max_print):
            s = "+ %04d - %s" % (i, t['fps'][ind_mst[i]])
            for kwd in kwds:
                s += '  %s' % t[kwd][ind_mst[i]]
            print(s)
    print("+ --------------------------------------------------")


def produce_master(t, method="median", imagetp='FLAT', slc=None,
                   ccdread_unit='adu'):
    """ a method to process master frames

    Parameters
    ----------
    t: astropy.table.Table
        the table of SONG observation
    method:
        the method adopted when combining frames
    imagetp: string
        {'BIAS', 'FLAT', 'FLATI2', 'THAR', 'THARI2'}
    slc: slice
        to denote the fraction of being used
    ccdread_unit: string/unit
        default is 'adu'

    Returns
    -------
    mst: ccdproc.CCDData
        the (combined) master frame

    """
    assert method in {'average', 'median'}

    # 1. produce ind of master frames

    ind_mst = np.where(t['IMAGETYP'] == imagetp)[0]
    # check for positive number of master frames
    try:
        assert len(ind_mst) > 0
    except AssertionError:
        raise IOError("There is no image of type %s!" % imagetp)

    # in default, combine all masters available
    if slc is not None:
        # combine a fraction of masters available
        assert isinstance(slc, slice)
        ind_mst = ind_mst[slc]
    # check for positive number of master frames
    try:
        assert len(ind_mst) > 0
    except AssertionError:
        raise IOError("There is no image of type %s! (slice is bad)" % imagetp)

    # 2. read master frames
    print("@SONG: trying really hard to produce the final %s frame!" % imagetp)
    for _ in t['fps'][ind_mst]:
        print("+ %s" % _)
    fps = ','.join(t['fps'][ind_mst])
    if len(ind_mst) == 1:
        mst = ccdproc.CCDData.read(fps, unit=ccdread_unit)
    else:
        mst = ccdproc.combine(fps, unit=ccdread_unit, method=method)

    return mst



# #!/usr/bin/env python2
# # -*- coding: utf-8 -*-
# """
# Created on Sat Dec  3 23:10:05 2016
#
# @author: cham
# """
#
# #%%
# import os
# import re
# import numpy as np
# import ccdproc
#
# from collections import OrderedDict
# from astropy.table import Table, Column, Row
# from hrs.logtable import log2table
# from hrs.aperture import find_apertures, combine_apertures, group_apertures, extract_1dspec, combine_flat, substract_scattered_light
# from hrs.calibration import fix_thar_sat_neg, thar_corr2d, interpolate_wavelength_shift, interpolate_order_shift, load_thar_temp
# import hrs
# #from hrs import *
# import hrs
# reload(hrs)
# from joblib import Parallel, delayed
#
#
# from scipy.interpolate import interp1d
# from skimage.filters import gaussian
#
# os.chdir('/hydrogen/song/star_spec/20161114/night/raw/')
#
# #%matplotlib qt
# from matplotlib import rcParams
#
# import glob
# from astropy.io import fits
# rcParams.update({'font.size':20})
# #%%
#
# """ scan fits header """
# def scan_fits_header(fps):
#
#     for i, fp in enumerate(fps):
#         print i, len(fps)
#         # read fits
#         hl = fits.open(fp)
#         hdr_dict = dict(hl[0].header)
#         # initiate table
#         if i == 0:
#             t = Table()
#             for k, v in hdr_dict.items():
# #                print k, v
#                 if type(v) is str:
#                     t.add_column(Column(data=np.array([]), dtype=object, name=k))
#                 else:
#                     t.add_column(Column(data=np.array([]), dtype=type(v), name=k))
#         # add rows
#         t.add_row(hdr_dict.values())
#     t.add_column(Column(fps, 'fps'), 0)
#     return t
#
#
# #fps_all = glob.glob('/hydrogen/song/star_spec/20161114/night/raw/*.fits')
#
# #fps_all = glob.glob('/hydrogen/song/star_spec/20161206/test/*.fits')
# fps_all = glob.glob('/hydrogen/song/star_spec/20161208/night/raw/*.fits')
# fps_all.sort()
# t = scan_fits_header(fps_all)
# t.show_in_browser()
#
# #%% look into mjd distribution
# fig = figure()
# plot(t['MJD-MID'][])
#
#
# #%% vis corr
# from mpl_toolkits.mplot3d import Axes3D
# fig = plt.figure(figsize=(16, 12))
# ax = fig.add_subplot(111, projection='3d')
#
# vx = np.arange(-20, 21)
# vy = np.arange(-50, 51)
# mx, my = np.meshgrid(vx, vy)
#
# ax.plot_surface(mx, my, corr[1])
#
# #%% vis thar
# from mpl_toolkits.mplot3d import Axes3D
# fig = plt.figure(figsize=(16, 12))
# ax = fig.add_subplot(111, projection='3d')
#
# vx = np.arange(2048)
# vy = np.arange(2048)
# mx, my = np.meshgrid(vx, vy)
#
# ax.plot_surface(mx, my, thar0)
#
#
# #from astropy.time import Time
# #t.add_column(Column(Time(t['FRAME'].data.astype(str)).mjd, 'MJD'))
#
# bias = fits.open('/hydrogen/song/star_spec/20161206/night/raw/s2_2016-12-07T00-06-00.fits')[0].data
# i1=0
# thar1 = (fits.open(fps[i1])[0].data.reshape(2048, 2048)).astype(float)
# i2=145
# thar2 = (fits.open(fps[i2])[0].data.reshape(2048, 2048)).astype(float)
# thar_corr2d(thar1, thar2, xtrim=(512, 1536), ytrim=(512, 1536), x_shiftmax=30, y_shiftmax=30)
#
# def par_corr2(thar1, i2, fps):
#     thar2 = (fits.open(fps[i2])[0].data.reshape(2048, 2048)).astype(float)
#     corr = thar_corr2d(thar1, thar2, xtrim=(512, 1536), ytrim=(512, 1536), x_shiftmax=30, y_shiftmax=30)
#     return corr
#
# corr_list = Parallel(n_jobs=20, verbose=10)(delayed(par_corr2)(thar1, i2, fps_all) for i2 in range(len(fps_all)))
#
# xshift=np.array([_[0][0][0] for _ in corr_list], dtype=int)
# yshift=np.array([_[0][1][0] for _ in corr_list], dtype=int)
# plot(xshift)
# plot(yshift)
#
#
# print("%s: %s, %s" % (fps[i2], corr[0][0][0], corr[0][1][0]))
#
# imshow(corr[1], interpolation='nearest')
# imshow(log10(thar1), interpolation='nearest')
#
#
#
# t.show_in_browser()
#
# plot(t['MJD-MID'],t['BVC'], 'o-')
#
#
#
#
# print t['IMAGETYP','EXPTIME','OBJECT','OBJ-RA','OBJ-DEC','I2POS']
# print np.unique(t['IMAGETYP'])
# np.count_nonzero
#
#
#
# #%%
# """ check Y shift """
#
# fps_all = glob.glob('/hydrogen/song/star_spec/20161206/night/raw/*.fits')
# fps_all.sort()
# t = scan_fits_header(fps_all)
# t.show_in_browser()
#
# ind_bias = t['IMAGETYP'] == 'BIAS'
# print("%s BIAS found" % np.sum(ind_bias))
# bias = ccdproc.combine(','.join(t['fps'][ind_bias]), unit='adu', method='average')
#
#
# ind_flat = t['IMAGETYP'] == 'FLAT'
# print("%s FLAT found" % np.sum(ind_flat))
# flat = ccdproc.combine(','.join(t['fps'][ind_flat]), unit='adu', method='median')
#
#
# ind_thar = t['IMAGETYP'] == 'THAR'
# print("%s THAR found" % np.sum(ind_thar))
# thar = ccdproc.combine(','.join(t['fps'][ind_thar]), unit='adu', method='median')
#
# """ rotation """
# def ccd_rot90(ccd, k=1):
#     return ccdproc.CCDData(np.rot90(ccd, k=k), unit=ccd.unit)
#
# bias_rot = ccd_rot90(bias)
# flat_rot = ccd_rot90(flat)
# thar_rot = ccd_rot90(thar)
#
# """ bias substraction """
# flat_rot_bias = ccdproc.subtract_bias(flat_rot, bias_rot)
# thar_rot_bias = ccdproc.subtract_bias(thar_rot, bias_rot)
#
#
# sub_thar = np.where(ind_thar)[0]
# i1=0
# thar1 = np.array(ccdproc.CCDData.read(fps_all[sub_thar[i1]], unit='adu')).astype(float)
#
# t['IMAGETYP'][sub_thar]
#
# for i2 in range(len(sub_thar)):
#     print i2
#     thar2 = np.array(ccdproc.CCDData.read(fps_all[sub_thar[i2]], unit='adu')).astype(float)
#     corr2d = thar_corr2d(thar1, thar2, xtrim=(512, 1536), ytrim=(512, 1536), x_shiftmax=30, y_shiftmax=30)
#     print corr2d[0]
#
# imshow(corr2d[1])
# imshow(thar1)
#
# #figure();imshow(bias)
# #figure();imshow(flat_rot)
# #figure();imshow(np.log10(thar))
# #figure();imshow(np.rot90(np.log10(thar)))
# #
# #type(t['IMAGETYP'][0])
#
# """ combine flat """
# flat_list = [flat_rot_bias]
#
# # find & combine & group apertures
# find_aps_param_dict = dict(start_col=200, max_drift=8, max_apwidth=12, n_pix_goodap=1000)
# ap_comb = hrs.combine_apertures(flat_list, n_jobs=10, find_aps_param_dict=find_aps_param_dict)
# cheb_coefs, ap_uorder_interp = group_apertures(ap_comb, start_col=1000, order_dist=10)
#
# # combine flat
# #flat_comb, flat_origin = combine_flat(flat_list, ap_uorder_interp, sat_count=45000, p=95)
# #flat_comb = ccdproc.CCDData(flat_comb, unit='adu')
# flat_comb = flat_rot_bias
#
# # scattered light substraction
# flat_comb_sl = substract_scattered_light(flat_comb, ap_uorder_interp, ap_width=13, shrink=.85)
# flat1d = extract_1dspec(flat_comb_sl, ap_uorder_interp, ap_width=10)[0]
#
# thar1d = extract_1dspec(thar_rot_bias, ap_uorder_interp, ap_width=10)[0]
#
# fig=figure()
# ax = fig.add_subplot(121)
# ax.imshow(np.log10(flat_rot_bias), interpolation='nearest')
# ax.plot(ap_uorder_interp.T, 'k')
# ax = fig.add_subplot(122)
# ax.imshow(np.log10(star_rot_bias), interpolation='nearest')
# ax.plot(ap_uorder_interp.T, 'k')
# ax.contour(np.log10(flat_rot_bias), c='w')
#
#
# fig=figure()
# imshow(np.log10(star_rot_bias), interpolation='nearest', cmap=cm.gray)
# plot(ap_uorder_interp.T, 'k')
# contour(np.log10(flat_rot_bias), c='k')
#
# figure()
# plot(flat_rot_bias[:, 1000])
# plot(star_rot_bias[:, 1000])
#
#
# figure()
# plot(flat1d.T)
#
#
# figure()
# icol=1000
# plot(flat_comb[:, icol])
# plot((flat_comb-flat_comb_sl)[:, icol])
#
#
# figure()
# for i in range(thar1d.shape[0]):
#     plot(thar1d[i]/np.max(thar1d[i])+i)
# #%%
# fits.HDUList([fits.PrimaryHDU(data=thar1d)]).writeto('../thar1d.fits', clobber=True)
#
# """ star """
# ind_star = t['IMAGETYP']=='STAR'
# n_star = np.sum(ind_star)
#
# for i in range(np.sum(ind_star)):
# #for i in range(10):
#     star = ccdproc.CCDData.read(t['fps'][np.where(ind_star)[0][i]], unit='adu')
#     # rotation
#     star_rot = ccd_rot90(star)
#
#     fig = figure(figsize=(8,8))
#     plt.imshow(np.log10(star_rot), interpolation='nearest')
#     plt.xlim(-0.5, 100.5)
#     plt.ylim(100.5, -0.5)
#     plt.grid(c='w',lw=1)
#     fig.savefig("../figs/%04d.png" % i)
#     plt.close(fig)
#
#     print(i)
#
#
# def compute_shifty(ccd, ap_uorder_interp, test_drift = np.arange(-12, 8)):
#     corr_drift = np.zeros_like(test_drift)
#     xcoord = np.arange(ccd.shape[1])
#     for i_drift,drift_ in enumerate(test_drift):
#         ap_drift = (ap_uorder_interp+drift_).astype(np.int)
#         this_corr = 0
#         for ap_drift_ in ap_drift:
#             this_corr += np.sum(star_rot[ap_drift_, xcoord])
#         corr_drift[i_drift] = this_corr
#     return corr_drift
#
#
# test_drift = np.arange(-12, 8)
# drifts_star = np.zeros((n_star))
# for i in range(np.sum(ind_star)):
#     #i = 1
#     star = ccdproc.CCDData.read(t['fps'][np.where(ind_star)[0][i]], unit='adu')
#     # rotation
#     star_rot = ccd_rot90(star)
#
#     corr_drift = compute_shifty(star_rot, ap_uorder_interp, test_drift = np.arange(-12, 8))
#     #smooth
#     corr_drift = np.convolve(corr_drift, [0.3, 0.4, 0.3], 'same')
#
#     this_drift = test_drift[np.argmax(corr_drift)]
#     drifts_star[i] = this_drift
#     print ("@Cham: drift of this image is ", this_drift)
#
#
# n_flat = np.sum(ind_flat)
# drifts_flat = np.zeros((n_flat))
# for i in range(np.sum(ind_flat)):
#     #i = 1
#     star = ccdproc.CCDData.read(t['fps'][np.where(ind_flat)[0][i]], unit='adu')
#     # rotation
#     star_rot = ccd_rot90(star)
#
#     corr_drift = compute_shifty(star_rot, ap_uorder_interp, test_drift = np.arange(-12, 8))
#     #smooth
#     corr_drift = np.convolve(corr_drift, [0.3, 0.4, 0.3], 'same')
#
#     this_drift = test_drift[np.argmax(corr_drift)]
#     drifts_flat[i] = this_drift
#     print ("@Cham: drift of this image is ", this_drift)
#
#
# #figure(); plot(corr_drift)
# fig=figure(figsize=(15, 6));
# ls, = plot(t['MJD-MID'][ind_star], drifts_star, 'bo-', ms=5)
# lf, = plot(t['MJD-MID'][ind_flat], drifts_flat, 'o', mfc='None', mec='r', ms=10, lw=2)
# legend((ls, lf), ['STAR', 'FLAT'], loc=2)
# xlabel('MJD-MID')
# ylabel("DRIFT/pixel (relative to stacked FLAT)")
# fig.tight_layout()
#
#
# figure();
# plot(t['MJD-MID'][ind_bias], 'go-')
# plot(t['MJD-MID'][ind_flat], 'ro-')
# plot(t['MJD-MID'][ind_star], 'bo-')
#
#
# #%%
# reload(hrs)
# fit_init = np.loadtxt('../database/fit_init.dat')
#
# lc_order = fit_init[:, 0]+85
# lc_coord = fit_init[:, 2]
# lc_thar = fit_init[:, 4]
#
# popt=np.repeat(np.array([[0,0,.1, 0.5, .5]]), len(lc_order),axis=0)
# pcov=np.repeat(np.array([[0,0,.1, 0.5, .5]]), len(lc_order),axis=0)
# # fit grating function
# poly_order = (3, 5)
# x_mini_lsq, ind_good_thar, scaler_coord, scaler_order, scaler_ml = hrs.fit_grating_equation(
#     lc_coord, lc_order, lc_thar, popt, pcov, poly_order=poly_order, max_dev_threshold=1., iter=False)
#
# # recover the wavelength (grid)
# grid_coord = np.repeat(np.arange(thar1d.shape[1]).reshape(1,-1), thar1d.shape[0], axis=0)
# grid_order = np.repeat(np.arange(thar1d.shape[0]).reshape(-1,1), thar1d.shape[1], axis=1)+85
# wave_fitted =  hrs.calibration.grating_equation_predict(grid_coord, grid_order, x_mini_lsq, poly_order, scaler_coord, scaler_order, scaler_ml)
#
#
# from mpl_toolkits.mplot3d import Axes3D
# fig = plt.figure(figsize=(16, 12))
# ax = fig.add_subplot(111, projection='3d')
# ax.plot(lc_coord, lc_order, lc_thar*lc_order, '.')
# ax.plot_surface(grid_coord, grid_order, wave_fitted*grid_order)
#
#
# #%%
# """ a better estimation of wavelength """
# reload(hrs)
#
# thar_list = np.loadtxt('/home/cham/PycharmProjects/hrs/hrs/calibration/iraf/thar.dat')
#
# #xcoord = np.repeat(np.arange(thar1d.shape[1]).reshape(1, -1), thar1d.shape[0], axis=0)
# wave_init = np.fliplr(np.loadtxt('../lfitr.csv', delimiter=','))
#
#
#
#
# #%%
# #wave_init = wave_fitted-10
#
# order_init = np.repeat(np.arange(thar1d.shape[0]).reshape(-1, 1), thar1d.shape[1], axis=1)+86
#
# # refine thar positions
# lc_coord, lc_order, lc_thar, popt, pcov, tcks = \
#     hrs.refine_thar_positions(wave_init, order_init, thar1d, thar_list, n_jobs=20, verbose=10)
#
# #figure();imshow(thar1d,aspect='auto')
# print np.sum(np.isfinite(lc_coord))
#
#
# #%%
# # fit grating function
# poly_order = (3, 5)
# x_mini_lsq, ind_good_thar, scaler_coord, scaler_order, scaler_ml = hrs.calibration.fit_grating_equation(
#     lc_coord, lc_order, lc_thar, popt, pcov, poly_order=poly_order, max_dev_threshold=1., iter=False)
# #x_mini_lsq, ind_good_thar, scaler_coord, scaler_order, scaler_ml = hrs.calibration.fit_grating_equation(
# #    lc_coord[ind_good_thar], lc_order[ind_good_thar], lc_thar[ind_good_thar], popt, pcov, poly_order=poly_order, max_dev_threshold=2., iter=False)
#
# # recover the wavelength (grid)
# grid_coord = np.repeat(np.arange(thar1d.shape[1]).reshape(1,-1), thar1d.shape[0], axis=0)
# grid_order = order_init
# wave_fitted =  hrs.calibration.grating_equation_predict(grid_coord, grid_order, x_mini_lsq, poly_order, scaler_coord, scaler_order, scaler_ml)
#
# # recover the wavelength (thar)
# lc_fitted = hrs.grating_equation_predict(lc_coord, lc_order, x_mini_lsq, poly_order, scaler_coord, scaler_order, scaler_ml)
# lc_fitted_diff = lc_fitted - lc_thar
# ind_good_thar = np.abs(lc_fitted_diff)<1
#
# np.sqrt(np.mean(np.square(lc_fitted_diff[ind_good_thar])))
# #%%
# """ residual check """
# fig = figure(figsize=(16,8));
# ax = fig.add_subplot(221)
# ax.plot(lc_order,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_order[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("Order")
# axis([61, 161, -5.2, 5.2])
#
# ax = fig.add_subplot(222)
# ax.plot(lc_coord,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_coord[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("CCD X Coordinate")
# xticks(np.arange(5)*1024)
# axis([0, 4096, -5.2, 5.2])
#
# ax = fig.add_subplot(212)
# ax.plot(lc_thar,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_thar[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("True ThAr wavelength")
# axis([3700, 10000, -5.2, 5.2])
#
# fig.tight_layout()
#
#
# figure()
# for i in range(wave_fitted.shape[0]):
#     plot(wave_fitted[i], thar1d[i]/np.max(thar1d[i]))
#
# #%%
# i = 0
# star = ccdproc.CCDData.read(t['fps'][np.where(ind_star)[0][i]], unit='adu')
# # rotation
# star_rot = ccd_rot90(star)
# # bias substraction
# star_rot_bias = ccdproc.subtract_bias(star_rot, bias_rot)
# # scatterd light
# star_comb_sl = substract_scattered_light(star_rot_bias, ap_uorder_interp+drifts_star[i], ap_width=8, shrink=.85)
# # extract 1D
# star1d = extract_1dspec(star_comb_sl, ap_uorder_interp+drifts_star[i], ap_width=10)[0]
# # de-blaze
# star1d_db = star1d/flat1d
# # scale
# star1d_dbsc = star1d_db/np.median(star1d_db, axis=1)[:, None]/2
#
# figure()
# imshow(star_comb_sl)
# plot(ap_uorder_interp.T, 'k')
#
# figure()
# for i in range(star1d_dbsc.shape[0]):
#     plot(star1d_dbsc[i]+i)
#
# figure()
# plot(star1d_db.T)
# plot(flat1d.T)
#
# figure()
# imshow(star_rot_bias)
#
# #%%
# %%time
# """ bias """
# ind_bias = logt['obj']=='bias'
# bias = ccdproc.combine(','.join(logt['filename'][ind_bias]), unit='adu', method='median')
#
# """ flat """
# ind_flat1_1 = (logt['obj']=='flat')*(logt['exp_time']==8)
# ind_flat1_2 = (logt['obj']=='flat')*(logt['exp_time']==9)
# ind_flat2_1 = (logt['obj']=='flat')*(logt['exp_time']==32)
# ind_flat2_2 = (logt['obj']=='flat')*(logt['exp_time']==36)
# ind_flat3 = (logt['obj']=='flat')*(logt['exp_time']==144)
#
# flat1_1 = ccdproc.combine(','.join(logt['filename'][ind_flat1_1]), unit='adu', method='median')
# flat1_2 = ccdproc.combine(','.join(logt['filename'][ind_flat1_2]), unit='adu', method='median')
# flat2_1 = ccdproc.combine(','.join(logt['filename'][ind_flat2_1]), unit='adu', method='median')
# flat2_2 = ccdproc.combine(','.join(logt['filename'][ind_flat2_2]), unit='adu', method='median')
# flat3 = ccdproc.combine(','.join(logt['filename'][ind_flat3]), unit='adu', method='median')
# flat1 = ccdproc.combine([flat1_1.multiply(9.).divide(8.), flat1_2], method='average')
# flat2 = ccdproc.combine([flat2_1.multiply(36.).divide(32.), flat2_2], method='average')
#
# flat1_bias = ccdproc.subtract_bias(flat1, bias)
# flat2_bias = ccdproc.subtract_bias(flat2, bias)
# flat3_bias = ccdproc.subtract_bias(flat3, bias)
#
# flat1_bias_trim = ccdproc.trim_image(flat1_bias[:, :4096])
# flat2_bias_trim = ccdproc.trim_image(flat2_bias[:, :4096])
# flat3_bias_trim = ccdproc.trim_image(flat3_bias[:, :4096])
#
#
# """ combine flat """
# flat_list = [flat1_bias_trim, flat2_bias_trim, flat3_bias_trim]
#
# # find & combine & group apertures
# ap_comb = combine_apertures(flat_list, n_jobs=10)
# cheb_coefs, ap_uorder_interp = group_apertures(ap_comb, start_col=2100, order_dist=10)
#
# # combine flat
# flat_comb, flat_origin = combine_flat(flat_list, ap_uorder_interp, sat_count=45000, p=95)
# flat_comb = ccdproc.CCDData(flat_comb, unit='adu')
#
# # scattered light substraction
# flat_comb_sl = substract_scattered_light(flat_comb, ap_uorder_interp, ap_width=10, shrink=.85)
# flat1d = extract_1dspec(flat_comb_sl, ap_uorder_interp, ap_width=7)[0]
#
#
# """ thar """
# ind_thar1 = (logt['obj']=='thar')*(logt['exp_time']==30)
# ind_thar2 = (logt['obj']=='thar')*(logt['exp_time']==60)
# ind_thar3 = (logt['obj']=='thar')*(logt['exp_time']==120)
#
# thar1 = ccdproc.combine(','.join(logt['filename'][ind_thar1]), unit='adu', method='average')
# thar2 = ccdproc.combine(','.join(logt['filename'][ind_thar2]), unit='adu', method='average')
# thar3 = ccdproc.CCDData.read(','.join(logt['filename'][ind_thar3]), unit='adu', method='average')
#
# thar1_bias = ccdproc.subtract_bias(thar1, bias)
# thar2_bias = ccdproc.subtract_bias(thar2, bias)
# thar3_bias = ccdproc.subtract_bias(thar3, bias)
#
# thar1_bias_trim = ccdproc.trim_image(thar1_bias[:, :4096])
# thar2_bias_trim = ccdproc.trim_image(thar2_bias[:, :4096])
# thar3_bias_trim = ccdproc.trim_image(thar3_bias[:, :4096])
#
# """ combine thar """
# thar_list = [thar1_bias_trim, thar2_bias_trim, thar3_bias_trim]
# thar_comb, thar_origin = combine_flat(thar_list, ap_uorder_interp, sat_count=45000, p=100)
# thar_comb = ccdproc.CCDData(thar_comb, unit='adu')
#
# """ extract 1d thar """
# #thar_comb_sl = substract_scattered_light(thar_comb, ap_uorder_interp, ap_width=10, shrink=.5)
# thar1d = extract_1dspec(thar_comb, ap_uorder_interp, ap_width=7)[0]
# thar1d_scaled = thar1d/flat1d
#
#
# #%%
# import matplotlib
# matplotlib.rcParams.update({'font.size': 20})
#
# fig = figure(figsize=(22, 6))
# imshow(log10(thar1d), vmin=2., vmax=4.5, aspect='auto', interpolation='nearest')
# xlabel('CCD X Coordinate')
# ylabel('Extracted Order Number')
# colorbar()
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/data/visualization_thar1d.pdf')
# plt.close(fig)
#
# fig = figure(figsize=(22, 6))
# imshow(spec1d_nm, vmin=0.85, vmax=1.05, aspect='auto', interpolation='nearest')
# xlabel('CCD X Coordinate')
# ylabel('Extracted Order Number')
# colorbar()
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/data/visualization_spec1d.pdf')
# plt.close(fig)
#
# #%%
# """ wavelength calibration """
#
#
# # laod template thar
# thar_temp_path = '/home/cham/PycharmProjects/hrs/hrs/calibration/thar_template/thar_temp_w20160120022t.fits'
# wave_temp, thar_temp, order_temp = load_thar_temp(thar_temp_path)
#
# wave_temp = np.loadtxt('/hydrogen/song/20150903/night/calib_master/wave.dat')
# thar_temp = np.loadtxt('/hydrogen/song/20150903/night/calib_master/thar.dat')
# wave_temp = np.rot90(wave_temp, 2)
# thar_temp = np.rot90(thar_temp, 2)
# thar_temp = np.fliplr(thar_temp)
#
# order_temp = np.repeat(np.arange(51).reshape(-1, 1), 2048, axis=1)+86
#
#
# # fix thar
# #thar1d_fixed = fix_thar_sat_neg(thar1d, arm=30, sat_count=500000)
# #thar_temp_fixed = fix_thar_sat_neg(thar_temp, arm=30, sat_count=500000)
#
#
# """ initial estimation of wavelength """
# shift, corr2d = thar_corr2d(thar1d, thar_temp, xtrim=(512, 512*3), ytrim=(20, 40), y_shiftmax = 1, x_shiftmax=50)
# print ("@Cham: the shift is ", shift)
# figure(); imshow(corr2d, interpolation='nearest', aspect='auto')
# figure(); imshow(np.log10(thar1d), interpolation='nearest', aspect='auto')
# figure(); imshow(np.log10(thar_temp), interpolation='nearest', aspect='auto')
# figure(); plot(thar1d[1]), plot(thar_temp[1][::-1])
# figure(); plot(wave_temp[0])
#
#
# wave_init = interpolate_wavelength_shift(wave_temp, shift, thar_temp, thar1d)
# order_init = interpolate_order_shift(order_temp, shift, thar1d)
#
#
#
# #%%
# fig=figure(figsize=(12, 3))
# imshow(corr2d, interpolation='nearest', aspect='auto', extent=(-20.5, 20.5, -3.5, 3.5))
# xlabel("X shift")
# ylabel("Y shift")
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_corr2d.pdf')
#
#
# #%%
# """ a better estimation of wavelength """
#
# thar_list = np.loadtxt('/home/cham/PycharmProjects/hrs/hrs/calibration/iraf/thar.dat')
#
# # refine thar positions
# lc_coord, lc_order, lc_thar, popt, pcov,tcks = \
#     hrs.calibration.refine_thar_positions(wave_init, order_init, thar1d, thar_list, n_jobs=20, verbose=10)
# #x_mini_lsq, ind_good_thar, scaler_coord, scaler_order, scaler_ml = hrs.calibration.fit_grating_equation(
# #    lc_coord[ind_good_thar], lc_order[ind_good_thar], lc_thar[ind_good_thar], popt, pcov, poly_order=poly_order, max_dev_threshold=2., iter=False)
# #figure();hist(popt[:,3]-lc_thar, np.arange(-10, 10, .1))
# #figure()
# #plot(lc_thar-popt[:, 3], popt[:, 4], '.')
# #ylim(-1, 5)
# # fit grating function
# poly_order = (3,4)
# x_mini_lsq, ind_good_thar, scaler_coord, scaler_order, scaler_ml = hrs.calibration.fit_grating_equation(
#     lc_coord, lc_order, lc_thar, popt, pcov, poly_order=poly_order, max_dev_threshold=3., iter=False)
# #x_mini_lsq, ind_good_thar, scaler_coord, scaler_order, scaler_ml = hrs.calibration.fit_grating_equation(
# #    lc_coord[ind_good_thar], lc_order[ind_good_thar], lc_thar[ind_good_thar], popt, pcov, poly_order=poly_order, max_dev_threshold=2., iter=False)
#
# # recover the wavelength (grid)
# grid_coord = np.repeat(np.arange(wave_init.shape[1]).reshape(1,-1), wave_init.shape[0], axis=0)
# grid_order = order_init
# wave_fitted =  hrs.calibration.grating_equation_predict(grid_coord, grid_order, x_mini_lsq, poly_order, scaler_coord, scaler_order, scaler_ml)
#
# # recover the wavelength (thar)
# lc_fitted = hrs.grating_equation_predict(lc_coord, lc_order, x_mini_lsq, poly_order, scaler_coord, scaler_order, scaler_ml)
# lc_fitted_diff = lc_fitted - lc_thar
#
# ind_good_thar = np.abs(lc_fitted_diff)<2
# #%%
# figure()
# plot(wave_fitted.T, wave_init.T-wave_fitted.T, '.')
#
# figure()
# plot(wave_init.T, thar1d.T, 'b')
# plot(wave_temp.T, thar_temp.T, 'r')
# plot(wave_fitted.T, thar1d.T+30000, 'c')
#
#
# figure()
# plot(wave_fitted.T, star1d_dbsc.T, '-')
# plot(wave_init.T, spec1d_nm.T+1, '-')
#
# #%%
# """ figures """
#
#
# """ fitted gaussian superposition """
# ww = np.arange(3000,10600,.01)
# ff = np.zeros_like(ww)
# for i,popt_ in enumerate(popt):
# #    if np.all(np.isfinite(popt_)):
#     if ind_good_thar[i]:
#         g = hrs.calibration.gauss_poly1(ww, 0., 0., popt_[2], popt_[3], popt_[4])
#         ff = ff+g
#         print i, ff[0], np.sum(g)
#
# figure(); plot(ww, ff,'r'); plot(wave_init.T, thar1d_fixed.T, 'b')
# figure(); plot(popt[:, 2], popt[:, 4], '.')
#
#
# """ line consistency check """
# matplotlib.rcParams.update({'font.size': 20})
# fig=figure(figsize=(8,6));
# l1 = plot(wave_init.reshape(104, 4096).T, spec1d_nm.T+.3, 'b')
# l2 = plot(gfitted_wave.reshape(104, 4096).T, spec1d_nm.T, 'r')
# xlim(5894.2,5896.5)
# ylim(0.05, 1.5)
# xlabel("Wavelength (A)")
#
# l = legend((l1[0], l2[0]), ['Initial guess', 'Refined'], loc=9)
# l.draw_frame=False
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_refine_thar_positions.pdf')
#
#
# """ residual check """
# fig = figure(figsize=(16,8));
# ax = fig.add_subplot(221)
# ax.plot(lc_order,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_order[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("Order")
# axis([61, 161, -5.2, 5.2])
#
# ax = fig.add_subplot(222)
# ax.plot(lc_coord,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_coord[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("CCD X Coordinate")
# xticks(np.arange(5)*1024)
# axis([0, 4096, -5.2, 5.2])
#
# ax = fig.add_subplot(212)
# ax.plot(lc_thar,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_thar[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("True ThAr wavelength")
# axis([3700, 10000, -5.2, 5.2])
#
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_calibration_residuals.pdf')
#
# """ residual check zoom in """
# fig = figure(figsize=(16,8));
# ax = fig.add_subplot(221)
# ax.plot(lc_order,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_order[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("Order")
# axis([61, 161, -0.1, 0.1])
#
# ax = fig.add_subplot(222)
# ax.plot(lc_coord,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_coord[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("CCD X Coordinate")
# xticks(np.arange(5)*1024)
# axis([0, 4096, -0.1, 0.1])
#
# ax = fig.add_subplot(212)
# ax.plot(lc_thar,lc_fitted-lc_thar, 'b+', alpha=.8)
# ax.plot(lc_thar[ind_good_thar],lc_fitted[ind_good_thar]-lc_thar[ind_good_thar], 'r+', alpha=.8)
# xlabel("True ThAr wavelength")
# axis([3700, 10000, -0.1, 0.1])
#
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_calibration_residuals_zoomin.pdf')
#
#
# """ wavelength chagne 3D """
# from mpl_toolkits.mplot3d import Axes3D
# fig = plt.figure(figsize=(16, 12))
# ax = fig.add_subplot(111, projection='3d')
# ax.plot_surface(grid_coord, grid_order, wave_fitted-wave_init, rstride=10, cstride=256, cmap=cm.jet)
# ax.set_xlim(-1, 4096)
# ax.set_xticks(np.arange(5)*1024)
# ax.set_xlabel("CCD X Coordinate")
# ax.set_ylabel("Order")
# ax.set_zlabel("Wavelength change (A)")
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_calibration_wchange_3d.pdf')
#
#
# """ wavelength change 2D """
# fig = plt.figure(figsize=(16, 4))
# ax = fig.add_subplot(111)
# ax.plot(wave_fitted.T, wave_fitted.T-wave_init.T, 'b-')
# ax.set_xlim(3700, 10000)
# #ax.set_xticks(np.arange(5)*1024)
# ax.set_ylabel("Wavelength Chagne (A)")
# ax.set_xlabel("Refined Wavelength (A)")
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_calibration_wchange_2d.pdf')
#
#
# #%%
# %%time
# """ read science images """
# # ########################################################################### #
# #    CPU times: user 58.7 s, sys: 4.05 s, total: 1min 2s
# #    Wall time: 59 s
# # ########################################################################### #
#
# """
# Master variables:
#     bias
#     ap_uorder_interp
#     flat1d
#     wave_fitted
# """
#
# # 1. read
# sci = ccdproc.CCDData.read(logt['filename'][41], unit='adu')
#
# # 2. BIAS correction
# sci_bias = ccdproc.subtract_bias(sci, bias)
#
# # 3. trim
# sci_bias_trim = ccdproc.trim_image(sci_bias[:, :4096])
#
# # 4. scattered light correction
# sci_bias_trim_sl = substract_scattered_light(sci_bias_trim, ap_uorder_interp, ap_width=10, shrink=.85)
#
# # 5. extract 1D spectra
# spec1d = extract_1dspec(sci_bias_trim_sl, ap_uorder_interp, ap_width=7)[0]
#
# # 6. de-blaze
# spec1d_db = spec1d/flat1d
#
# # 7. roughly scale to 1.
# spec1d_nm = spec1d_db/np.percentile(spec1d_db, 95, axis=1)[:, None]
#
# figure(); plot(wave_fitted.T, spec1d_nm.T)
#
# #figure();plot(wave_fitted.T, np.sqrt(spec1d).T)
# #figure();plot(wave_fitted.T, np.sqrt(flat1d).T)
# #%%
# scattered_light = sci_bias_trim-sci_bias_trim_sl
# fig = figure(figsize=(16, 8))
# plot(sci_bias_trim[:, 2100])
# plot(scattered_light[:, 2100], 'r')
# xlabel("CCD Y coordinate")
# ylabel("CCD counts")
# ylim(-100, 21000)
# xlim(0, 4096)
# xticks(np.arange(5)*1024)
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_scattered_background.pdf')
#
#
#
# wave_fitted_masked = wave_fitted
# wave_fitted_masked = np.where(np.log10(flat1d)<3.2, np.nan, wave_fitted_masked)
#
# fig = figure(figsize=(20, 5))
# plot(wave_fitted_masked.T, spec1d_nm.T)
# xlim(6500, 6700)
# ylim(-0.05, 1.15)
# xlabel("Wavelength (A)")
# ylabel("De-blazed & scaled flux")
# fig.tight_layout()
# fig.savefig('/home/cham/PycharmProjects/hrs/hrs/figures/visualization_deblazed_spectrum.pdf')
#
# #%%
# figure(); imshow(flat1d, aspect='auto')
# figure();  imshow(log10(flat1d), aspect='auto', interpolation='nearest')
#
# #%%
#
# figure();  imshow(log10(flat1d)>3.5, aspect='auto', interpolation='nearest')
# #%%



# coding: utf-8

# In[1]:

from song import measure_xdrift


# In[2]:

dp = '/hydrogen/song/star_spec/20170102/night/raw/'


# In[3]:

t = measure_xdrift.scan_files(dp)


# In[4]:

t2, fig = measure_xdrift.check_xdrift(t)


# In[5]:

fig


# In[6]:

fig.savefig('/hydrogen/song/figs/Xdrift_20161206.svg')


# In[7]:

t.write('/hydrogen/song/figs/file_catalog_20161206.fits')




# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Fri Nov 25 12:53:24 2016

Modifications
-------------
-

Aims
----
- utils for apertures

"""

import numpy as np
from joblib import Parallel, delayed
from scipy.interpolate import interp1d
from skimage.filters import gaussian
import ccdproc


# ################################# #
#      find starting points
# ################################# #

def find_local_maximas(pixels, n_adj=2, n_smooth=1, n_sep=1):
    """ find all local maximas

    Parameters
    ----------
    pixels:
        a slice of CCD pixels
    n_adj:
        number of adjacent pixels for maximas
    n_smooth:
        running mean smooth window width
    n_sep:
        the separation since last maxima

    Returns
    -------
    ind of maximas

    """
    pixels = np.array(pixels).flatten()
    n_pix = len(pixels)

    ind_max = np.zeros_like(pixels, dtype=bool)
    # smooth
    for i in np.arange(n_smooth, n_pix - n_smooth):
        pixels[i] = np.mean(np.array(pixels[i - n_smooth:i + 1 + n_smooth]))
    # find maximas
    for i in np.arange(n_adj, n_pix - n_adj):
        ind_max[i] = np.argmax(
            np.array(pixels[i - n_adj:i + 1 + n_adj])) == n_adj and np.sum(
            ind_max[i - n_sep:i]) < 1

    return ind_max


def find_local_minimas(pixels, n_adj=2, n_smooth=1, n_sep=10):
    """ find all local minimas

    Parameters
    ----------
    pixels:
        a slice of CCD pixels
    n_adj:
        number of adjacent pixels for minimas
    n_smooth:
        running mean smooth window width
    n_sep:
        the separation since last minima

    Returns
    -------
    ind of minimas

    """
    pixels = np.array(pixels).flatten()
    n_pix = len(pixels)

    ind_min = np.zeros_like(pixels, dtype=bool)
    # smooth
    for i in np.arange(n_smooth, n_pix - n_smooth):
        pixels[i] = np.mean(np.array(pixels[i - n_smooth:i + 1 + n_smooth]))
    # find minimas
    for i in np.arange(n_adj, n_pix - n_adj):
        ind_min[i] = np.argmin(
            np.array(pixels[i - n_adj:i + 1 + n_adj])) == n_adj and np.sum(
            ind_min[i - n_sep:i]) < 1

    return ind_min


def find_mmax_mmin(im, start_col=2100, n_adj=7, n_smooth=3, n_sep=5):
    """ find reasonable maximas and minimas (cross-kick)

    Parameters
    ----------
    im
    start_col

    Returns
    -------

    """
    start_col_slice = np.sum(im[:, start_col][:, None], axis=1)

    imax = find_local_maximas(start_col_slice, n_adj=n_adj, n_smooth=n_smooth,
                              n_sep=n_sep)
    imin = find_local_minimas(start_col_slice, n_adj=n_adj, n_smooth=n_smooth,
                              n_sep=n_sep)

    # no pixel could be both max&min
    try:
        assert np.sum(imax * imin) == 0
    except:
        raise (ValueError("@Cham: imax*imin != 0"))

    # cross kick
    smax = np.where(imax)[0]
    smin = np.where(imin)[0]
    smmax = []
    smmin = []
    for i in range(len(smax) - 1):
        imin_this = imin[smax[i]:smax[i + 1]]    # ind array for local mins
        smin_this = np.where(imin_this)[0]       # sub array for local mins
        if len(smin_this) == 1:
            smmin.append(smin_this[0] + smax[i])
        elif len(smin_this) > 1:
            # more than 1 local minimas, find the minimal minima
            smmin.append(smin_this[np.argmin(
                start_col_slice[smax[i]:smax[i + 1]][smin_this])] + smax[i])
    # smin = smmin
    for i in range(len(smmin) - 1):
        imax_this = imax[smmin[i]:smmin[i + 1]]  # ind array for local maxs
        smax_this = np.where(imax_this)[0]       # sub array for local maxs
        if len(smax_this) == 1:
            smmax.append(smax_this[0] + smmin[i])
        elif len(smax_this) > 1:
            # more than 1 local maximas, find the maximal maxima
            smmax.append(smax_this[np.argmax(
                start_col_slice[smmin[i]:smmin[i + 1]][smax_this])] + smmin[i])
    print("@Cham: %s possible apertures found (exciting)!" % len(smmax))
    return smmax, smmin


# ################################# #
#      find apertures
# ################################# #

def find_apertures(im, start_col=2100, max_drift=5, max_apwidth=10,
                   n_pix_goodap=1500, n_adj=7, n_smooth=3, n_sep=5, c=3):
    """ find apertures from image

    Parameters
    ----------
    im:
        image
    start_col:
        start column
    max_drift:
        max_drift in finding an aperture
    max_apwidth:
        local comparison width
    n_pix_goodap:
        a good aperture should be more than this number of pixels

    Returns
    -------
    ymmax_goodap: ndarray
        the y-pixel values for good apertures
    """
    # gaussian smooth if c>0
    if c > 0:
        im = ccdproc.CCDData(gaussian(im, sigma=(c, 0)), unit=im.unit)

    # find max & min
    smmax, smmin = find_mmax_mmin(im, start_col=start_col, n_adj=n_adj,
                                  n_smooth=n_smooth, n_sep=n_sep)

    # initialize results
    ymmax = np.zeros((len(smmax), im.shape[1]))
    ymmax[:, start_col] = smmax
    ymmin = np.zeros((len(smmin), im.shape[1]))
    ymmin[:, start_col] = smmin

    # tracing apertures
    for i_ap in range(ymmax.shape[0]):
        for i_col in np.arange(start_col + 1, im.shape[1]):
            y0 = ymmax[i_ap, i_col - 1]
            y1 = np.argmax(im[np.max((0, y0 - max_apwidth)):np.min(
                (im.shape[0], y0 + 1 + max_apwidth)), i_col].data *
                           im[np.max((0, y0 - max_apwidth)):np.min(
                               (im.shape[0], y0 + 1 + max_apwidth)),
                           i_col - 1].data) + y0 - max_apwidth
            if np.abs(y1 - y0) < max_drift:
                # good ap, continue
                ymmax[i_ap, i_col] = y1
            else:
                break
        for i_col in np.arange(start_col - 1, 0, -1):
            y0 = ymmax[i_ap, i_col + 1]
            y1 = np.argmax(im[np.max((0, y0 - max_apwidth)):np.min(
                (im.shape[0], y0 + 1 + max_apwidth)), i_col].data *
                           im[np.max((0, y0 - max_apwidth)):np.min(
                               (im.shape[0], y0 + 1 + max_apwidth)),
                           i_col + 1].data) + y0 - max_apwidth
            if np.abs(y1 - y0) < max_drift:
                # good ap, continue
                ymmax[i_ap, i_col] = y1
            else:
                break
        print("@Cham: tracing aperture [%s] " % i_ap)
    #
    # for i_ap in range(ymmin.shape[0]):
    #     for i_col in np.arange(start_col + 1, im.shape[1]):
    #         y0 = ymmin[i_ap, i_col - 1]
    #         y1 = np.argmin(im[np.max((0, y0 - max_apwidth)):np.min(
    #             (im.shape[0], y0 + 1 + max_apwidth)), i_col].data *
    #                        im[np.max((0, y0 - max_apwidth)):np.min(
    #                            (im.shape[0], y0 + 1 + max_apwidth)),
    #                        i_col - 1].data) + y0 - max_apwidth
    #         if np.abs(y1 - y0) < max_drift:
    #             # good ap, continue
    #             ymmin[i_ap, i_col] = y1
    #         else:
    #             break
    #     for i_col in np.arange(start_col - 1, 0, -1):
    #         y0 = ymmin[i_ap, i_col + 1]
    #         y1 = np.argmin(im[np.max((0, y0 - max_apwidth)):np.min(
    #             (im.shape[0], y0 + 1 + max_apwidth)), i_col].data *
    #                        im[np.max((0, y0 - max_apwidth)):np.min(
    #                            (im.shape[0], y0 + 1 + max_apwidth)),
    #                        i_col + 1].data) + y0 - max_apwidth
    #         if np.abs(y1 - y0) < max_drift:
    #             # good ap, continue
    #             ymmin[i_ap, i_col] = y1
    #         else:
    #             break
    #     print i_ap

    ind_goodap = np.sum(ymmax > 0, axis=1) > n_pix_goodap
    ymmax_goodap = ymmax[ind_goodap, :]
    print("@Cham: number of good aps (max) = %s " % np.sum(ind_goodap))

    #    ind_goodap = np.sum(ymmax>0, axis=1)>1000
    #    ymmin_goodap = ymmin[ind_goodap, :]
    #    print("@Cham: gumber of good aps (min)", np.sum(ind_goodap))

    return ymmax_goodap  # , ymmin_goodap


# ################################# #
#      combine & group apertures
# ################################# #

def combine_apertures(imlist, n_jobs=10, find_aps_param_dict=None):
    """ combine apertures found from different FLAT images

    Parameters
    ----------
    imlist: list
        list of FLAT
    find_aps_param_dict:
        the Parameters used in *find_apertures*
    n_jobs:
        n_jobs for finding apertures in parallel

    Returns
    -------
    ap_combine:
        the y-coordinates of good apertures found from FLAT list

    """
    if find_aps_param_dict is None:
        find_aps_param_dict = dict(start_col=2100, max_drift=9, max_apwidth=13,
                                   n_pix_goodap=1000, n_adj=7, n_smooth=3,
                                   n_sep=5, c=3)
    ap_list = Parallel(n_jobs=n_jobs, verbose=2)(
        delayed(find_apertures)(im, **find_aps_param_dict) for im in imlist)
    print("@Cham: the numbers of apertures found are: ",
          [ap_.shape[0] for ap_ in ap_list])
    ap_combine = np.vstack(ap_list)
    return ap_combine


def group_apertures(ap_comb, start_col=2100, order_dist=10):
    """ group combined apertures to unique apertures

    Parameters
    ----------
    ap_comb:
        combined apertures
    start_col:
        the starting column number
    order_dist:
        the typical distance between orders

    Returns
    -------
    cheb_coefs: ndarray
        the chebyshev polynomial coefs
    ap_uorder_interp: ndarray
        the y-coordinates of unique orders

    """
    # extract shape
    naps = ap_comb.shape[0]
    npix_disp = ap_comb.shape[1]

    # initialize
    x = np.arange(npix_disp)
    yi_start_col = np.zeros(naps)

    # rough chebyshev fit & interpolate
    for i in range(naps):
        y = ap_comb[i]
        c = np.polynomial.chebyshev.chebfit(x, y, 2, w=y > 0)
        yi_start_col[i] = np.polynomial.chebyshev.chebval(x[start_col], c)
    yi_start_col_sorted = np.sort(yi_start_col)
    yi_start_col_sortarg = np.argsort(yi_start_col)

    # find unique orders
    order_arg_list = []
    c = 0
    for i_uorder in range(naps):
        this_order_yi = yi_start_col_sorted[c]
        this_order_ind = np.abs(
            yi_start_col_sorted - this_order_yi) < order_dist
        this_order_arg = yi_start_col_sortarg[this_order_ind]
        order_arg_list.append(this_order_arg)
        c += len(this_order_arg)
        if c >= len(yi_start_col_sorted):
            break

    # fit & extrapolate
    n_uorder = len(order_arg_list)
    cheb_coefs = []
    ap_uorder_interp = np.zeros((n_uorder, npix_disp))
    for i_uorder in range(n_uorder):
        n_suborder = len(order_arg_list[i_uorder])
        x_data = np.repeat(x[:, None].T, n_suborder, axis=0).flatten()
        y_data = ap_comb[order_arg_list[i_uorder], :].flatten()
        c = np.polynomial.chebyshev.chebfit(x_data, y_data, 2, w=y_data > 0)
        cheb_coefs.append(c)
        ap_uorder_interp[i_uorder] = np.polynomial.chebyshev.chebval(x, c)

    print("@Cham: %s unique orders found!" % n_uorder)
    return cheb_coefs, ap_uorder_interp


# ################################# #
#      extract 1d spectra
# ################################# #

def extract_1dspec(im, ap_uorder_interp, ap_width=7):
    naps = ap_uorder_interp.shape[0]
    npix = ap_uorder_interp.shape[1]
    ap_int = ap_uorder_interp.astype(int)

    spec = np.zeros((naps, npix), dtype=float)
    sat_mask = np.zeros((naps, npix), dtype=bool)

    for i in range(naps):
        for j in range(npix):
            yc = ap_int[i, j]
            data = im[yc - ap_width:yc + ap_width + 1, j]
            spec[i, j] = np.sum(data)
            sat_mask[i, j] = np.any(data > 60000)
        print("@Cham: *extract_1dspec* extracting 1d spec for order [%s]" % i)

    return spec, sat_mask


# ################################# #
#      find ind for all orders
# ################################# #

def find_ind_order(ap_uorder_interp, ccd_shape, edge_len=(10, 20)):
    # find bounds for apertures
    ap_uorder_bounds = np.vstack(
        (ap_uorder_interp[0] - edge_len[0],
         np.diff(ap_uorder_interp, axis=0) * .5 + ap_uorder_interp[:-1],
         ap_uorder_interp[-1] + edge_len[1]))

    # initialize ind_order
    ind_order = np.zeros(ccd_shape, dtype=int)

    # generate coordinates
    x, y = np.arange(ccd_shape[1]), np.arange(ccd_shape[0])
    mx, my = np.meshgrid(x, y)

    # for each apertures
    for i_order in range(ap_uorder_bounds.shape[0] - 1):
        this_order_bound_l = ap_uorder_bounds[i_order, :].reshape(1, -1)
        this_order_bound_u = ap_uorder_bounds[i_order + 1].reshape(1, -1)
        ind_this_order = np.logical_and(my > this_order_bound_l,
                                        my < this_order_bound_u)
        ind_order[ind_this_order] = i_order
        print ("@Cham: marking pixels of order [%s]" % i_order)
    # for edges
    this_order_bound_l = -1
    this_order_bound_u = ap_uorder_bounds[0].reshape(1, -1)
    ind_this_order = np.logical_and(my > this_order_bound_l,
                                    my < this_order_bound_u)
    ind_order[ind_this_order] = -1

    this_order_bound_l = ap_uorder_bounds[-1].reshape(1, -1)
    this_order_bound_u = ccd_shape[0]
    ind_this_order = np.logical_and(my > this_order_bound_l,
                                    my < this_order_bound_u)
    ind_order[ind_this_order] = -2

    return ind_order


# ############################################### #
#      combine flat according to max counts
# ############################################### #

def combine_flat(flat_list, ap_uorder_interp, sat_count=50000, p=95):
    """ combine flat according to max value under sat_count

    Parameters
    ----------
    flat_list
    ap_uorder_interp
    sat_count
    p

    Returns
    -------

    """
    # find ind_order
    ind_order = find_ind_order(ap_uorder_interp, flat_list[0].shape)

    # unique orders
    uorder = np.unique(ind_order)
    uorder_valid = uorder[uorder >= 0]

    # find max for each order
    im_max = np.vstack(
        [find_each_order_max(im_, ind_order, p) for im_ in flat_list])

    # combine flat
    flat_final = np.zeros_like(flat_list[0])
    flat_origin = np.zeros_like(flat_list[0], dtype=int)
    for uorder_ in uorder:
        # find pixels of this uorder
        ind_this_order = ind_order == uorder_
        # which one should be used for this uorder
        i_im = find_max_under_saturation(im_max[:, uorder_],
                                         sat_count=sat_count)
        # fill the findal image with data from that image
        flat_final = np.where(ind_this_order, flat_list[i_im], flat_final)
        flat_origin[ind_this_order] = i_im
        print("@Cham: filling data: image [%s] --> uorder [%s]" %
              (i_im, uorder_))

    return flat_final, flat_origin


def find_max_under_saturation(max_vals, sat_count=45000):
    """ find the image with maximum value under saturation count number

    Parameters
    ----------
    max_vals
    sat_count

    Returns
    -------
    the ID of max value under sat_count

    """
    if np.any(max_vals < sat_count):
        asort = np.argsort(max_vals)
        for i in range(1, len(asort)):
            if max_vals[asort[i]] >= sat_count:
                return asort[i - 1]
        return asort[i]
    else:
        return np.argmin(max_vals)


# This is actually not used.
# I've implemented *find_each_order_max* for extracting info
# for combining FLAT
def extract_ap_ridge_counts(im, ap_interp):
    npix = im.shape[1]
    naps = ap_interp.shape[0]

    # float -> int
    ap_int = ap_interp.astype(int)

    spec = np.zeros(ap_int.shape, dtype=float)
    sat_mask = np.zeros(ap_int.shape, dtype=bool)
    for j in range(npix):
        for i in range(naps):
            yc = ap_int[i, j]
            data = im[yc, j]
            spec[i, j] = data.data
    return spec, sat_mask


def find_each_order_max(im, ind_order, p=95):
    """ find the max for each order

    Actually the max value is affected by cosmic rays.
    So, finding the 99/95/90 percentiles could be more useful.


    Parameters
    ----------
    im:
        FLAT image
    ind_order:
        ind of apertures
    p: float
        percentile

    Returns
    -------
    max_list: ndarray


    """
    uorder = np.unique(ind_order)
    uorder_valid = uorder[uorder >= 0]

    max_list = np.zeros_like(uorder_valid)
    for each_uorder in uorder_valid:
        max_list[each_uorder] = np.percentile(
            im[np.where(ind_order == each_uorder)], p)
        print("@Cham: finding maximum value for order [%s]" % each_uorder)

    return max_list


# ################################# #
#      scatterd light correction
# ################################# #

def substract_scattered_light(im, ap_uorder_interp, ap_width=10,
                              shrink=.85):  # 10 should be max
    ind_inter_order = find_inter_order(im, ap_uorder_interp, ap_width=ap_width)

    im_scattered_light = np.zeros_like(im)

    # interpolation
    x = np.arange(im.shape[0])
    print(
        "@Cham: *substract_scattered_light* interpolating scattered light ...")
    for i_col in range(im.shape[1]):
        w = ind_inter_order[:, i_col]
        y = im[:, i_col]
        I = interp1d(x[w], y[w], 'linear')
        im_scattered_light[:, i_col] = I(x)
    # smooth
    print("@Cham: *substract_scattered_light* smoothing scattered light ...")
    im_scattered_light_smooth = gaussian(im_scattered_light * shrink, (10, 8))
    return im - im_scattered_light_smooth


def find_inter_order(im, ap_uorder_interp, ap_width=10):
    ind_inter_order = np.ones_like(im, dtype=bool)

    # generate coordinates
    x, y = np.arange(im.shape[1]), np.arange(im.shape[0])
    mx, my = np.meshgrid(x, y)

    # for each apertures
    for i_order in range(ap_uorder_interp.shape[0] - 1):
        this_order_ridge = ap_uorder_interp[i_order, :].reshape(1, -1)
        ind_this_order = np.logical_and(my > this_order_ridge - ap_width,
                                        my < this_order_ridge + ap_width)
        ind_inter_order = np.where(ind_this_order, False, ind_inter_order)
        print ("@Cham: *find_inter_order* marking pixels of order [%s]" %
               i_order)
    return ind_inter_order



from distutils.core import setup


if __name__ == '__main__':
    setup(
        name='hrs',
        version='1.1.9',
        author='Bo Zhang',
        author_email='bozhang@nao.cas.cn',
        # py_modules=['hrs'],
        description='High Resolution Spectrograph (2.16m) Reduction pipeline.',  # short description
        license='New BSD',
        # install_requires=['numpy>=1.7','scipy','matplotlib','nose'],
        url='http://github.com/hypergravity/hrs',
        classifiers=[
            "Development Status :: 6 - Mature",
            "Intended Audience :: Science/Research",
            "License :: OSI Approved :: BSD License",
            "Operating System :: OS Independent",
            "Programming Language :: Python :: 2.7",
            "Topic :: Scientific/Engineering :: Astronomy",
            "Topic :: Scientific/Engineering :: Physics"],
        package_dir={'hrs/': ''},
        packages=['hrs', 'song'],
        # package_data={'hrs/data': [''],
        #               "":          ["LICENSE"]},
        include_package_data=True,
        requires=['numpy', 'scipy', 'matplotlib', 'astropy', 'skimage',
                  'joblib', 'ccdproc', 'tqdm']
    )




"""
an example of extracting 1d spectra for SONG

"""

from song import extract
from song import measure_xdrift
from song import ccdproc_mod
from song import utils
import hrs

from matplotlib import rcParams
rcParams.update({'font.size':20})
#%%

reload(extract)
reload(measure_xdrift)


#%%
#%pylab qt
#%matplotlib qt
#%load_ext autoreload
#%autoreload 2


#%%

# specify directory
dp = '/hydrogen/song/star_spec/20170102/night/raw/'

# scan fits files
t = measure_xdrift.scan_files(dp)

# measure cross-order drift
t2, fig = measure_xdrift.check_xdrift(t)
f90 = utils.scan_flux90(t, n_jobs=20, unit='adu')



flat = extract.produce_master(t2, method='average', imagetp='FLAT', slc=slice(0, 10)).rot90(1)
bias = extract.produce_master(t2, method='average', imagetp='BIAS', slc=slice(0, 10)).rot90(1)
flat_bias = ccdproc_mod.subtract_bias(flat, bias)

star = extract.produce_master(t2, method='median', imagetp='STAR', slc=slice(0, 1)).rot90(1)
star_bias = ccdproc_mod.subtract_bias(star, bias)

thar = extract.produce_master(t2, method='median', imagetp='THAR', slc=slice(0, 1)).rot90(1)
thar_bias = ccdproc_mod.subtract_bias(thar, bias)

extract.list_image(t2, imagetp='STAR', kwds=['OBJECT'])



#%%
""" VCUT """
flat216 = ccdproc_mod.CCDData.read('/town/HRS/20161110/20161110006.fits', unit='adu')
thar216 = ccdproc_mod.CCDData.read('/town/HRS/20161110/201611100039.fits', unit='adu')
star216 = ccdproc_mod.CCDData.read('/town/HRS/20161110/201611100052.fits', unit='adu')

fig = figure()
ax = fig.add_subplot(211)
ax.plot(flat_bias[:,1024])
ax.plot(thar_bias[:,1024])
ax.plot(star_bias[:,1024])

ax = fig.add_subplot(212)
ax.plot(flat216[:,2048])
ax.plot(thar216[:,2048])
ax.plot(star216[:,2048])

#%%

#extract.list_image(t, 'FLAT')

# find apertures
find_aps_param_dict = dict(start_col=440, max_drift=8, max_apwidth=10,
                           n_pix_goodap=100, n_adj=10, n_smooth=1, n_sep=3, c=5)
ap_comb = hrs.combine_apertures([flat_bias], n_jobs=1, find_aps_param_dict=find_aps_param_dict)
cheb_coefs, ap_uorder_interp = hrs.group_apertures(ap_comb, start_col=1024, order_dist=7)

# extract 1d FLAT & STAR & THAR
flat_bias_sl = hrs.substract_scattered_light(flat_bias, ap_uorder_interp, ap_width=10, shrink=.85)
flat1d = hrs.extract_1dspec(flat_bias_sl, ap_uorder_interp, ap_width=8)[0]

star_bias_sl = hrs.substract_scattered_light(star_bias, ap_uorder_interp, ap_width=10, shrink=.85)
star1d = hrs.extract_1dspec(star_bias_sl, ap_uorder_interp, ap_width=8)[0]

thar_bias_sl = hrs.substract_scattered_light(thar_bias, ap_uorder_interp, ap_width=10, shrink=.85)
thar1d = hrs.extract_1dspec(thar_bias_sl, ap_uorder_interp, ap_width=8)[0]



thar_temp_path = '/home/cham/PycharmProjects/hrs/hrs/calibration/thar_template/thar_temp_w20160120022t.fits'
wave_temp, thar_temp, order_temp = load_thar_temp(thar_temp_path)


star1d_dblz = star1d/flat1d

#%%
for i in range(star1d_dblz.shape[0]):
    plot(star1d_dblz[i, :]/np.median(star1d_dblz[i, :])+i*1)


#%%
imshow(flat, interpolation='nearest')
plot(ap_comb.T, 'k-')
plot(ap_uorder_interp.T, 'w-')




# -*- coding: utf-8 -*-
"""

Author
------
Bo Zhang

Email
-----
bozhang@nao.cas.cn

Created on
----------
- Wed Nov 23 17:37:24 2016

Modifications
-------------
-

Aims
----
- utils for CCD image processing

"""

from astropy.time import Time
from astropy.io import fits


class Frame(object):
    """ a basic class for constructing an image frame """
    filepath = ''
    type_ = ''

    header = None
    data = None
    read_status = False

    exp_start = Time('2000-01-01T00:00:00.000000000')
    exp_stop = Time('2000-01-01T00:00:01.000000000')
    exp_time = exp_stop - exp_start

    def __init__(self, filepath, type_='sci'):
        """ constructor of Frame instances

        Parameters
        ----------
        filepath: string
            file path
        type_: string
            { 'bias' | 'flat' | 'thar' | 'sci' }

        """
        self.filepath = filepath
        self.type_ = type_
        self.read_status = False

    def read(self):
        """ read data """
        hl = fits.open(self.filepath)
        self.header = hl[0].header
        self.data = hl[0].data
        self.read_status = True
        print('@Cham: data read! [%s]' % self.filepath)

    @property
    def pprint(self):
        print("--------------------------------------------------------------")
        print("    file: %s " % self.filepath)
        print("    type: %s " % self.type_)
        if self.read_status:
            print("  header: ")
            print(self.header.__repr__())
            print("    type: ")
            print(self.data.__repr__())
        print("--------------------------------------------------------------")
        pass


def test():
    import os
    os.chdir('/town/HRS/20161110')
    f = Frame('./20161110001.fits', 'flat')
    f.read()
    # print f.data
    # print f.data.shape
    # print f.header
    f.pprint

if __name__ == '__main__':
    test()
